package speiger.src.coreengine.rendering.utils;

import speiger.src.coreengine.utils.counters.averager.Averager;

public class AllocationTracker
{
	public static final AllocationTracker INSTANCE = new AllocationTracker();
	
	long lastTime = 0;
	long lastMemory = 0L;
	long addedCPUMemory = 0L;
	long addedGPUMemory = 0L;
	Averager cpuAllocation = new Averager(20);
	Averager gpuAllocation = new Averager(20);
	
	public void update()
	{
		long newTime = System.currentTimeMillis();
		if(newTime - lastTime < 1000)
		{
			updateAllocation();
			return;
		}
		lastTime = newTime;
		updateAllocation();
		cpuAllocation.addEntry(addedCPUMemory);
		addedCPUMemory = 0L;
		gpuAllocation.addEntry(addedGPUMemory);
		addedGPUMemory = 0L;
	}
	
	public void addGPUBytes(long amount)
	{
		addedGPUMemory += amount;
	}
	
	private void updateAllocation()
	{
		long memory = getMemory();
		if(memory > lastMemory)
		{
			addedCPUMemory += (memory - lastMemory);
		}
		lastMemory = memory;
	}
	
	private long getMemory()
	{
	    Runtime runtime = Runtime.getRuntime();
	    return runtime.totalMemory() - runtime.freeMemory();
	}
	
	public long getCPUAllocatedBytes()
	{
		return cpuAllocation.getAverage();
	}
	
	public long getGPUAllocationBytes()
	{
		return gpuAllocation.getAverage();
	}
}
